/// Top member of the Common Printer Doc format
abstract class Printerble {
  /// should return [Map<String,dynamic>] for platform specific code.
  Map<String, dynamic> toMap();

  const Printerble();
}

/// Abstract class for common Text properties for print.
/// [Size] is the font size type supported by the printer SDK.
abstract class CoreText<Size> extends Printerble {
  /// [isBold] boolean property to make Text Bold.
  final bool isBold;
  final bool isItalic;
  final Size size;
  final String align;

  const CoreText({
    required this.size,
    this.isBold = false,
    this.isItalic = false,
    this.align = 'LEFT',
  }) : super();

  @override
  Map<String, dynamic> toMap() {
    return {
      "size": size,
      "isBold": isBold,
      "isItalic": isItalic,
    };
  }
}

class Line<Size> extends CoreText<Size> {
  static const String type = "LINE";

  final String text;

  const Line({
    required this.text,
    bool isBold = false,
    bool isItalic = false,
    String align = 'LEFT',
    required Size size,
  }) : super(
          size: size,
          align: align,
          isBold: isBold,
          isItalic: isItalic,
        );

  @override
  Map<String, dynamic> toMap() {
    return {
      "type": type,
      "text": text,
      ...super.toMap(),
    };
  }
}

/// TWO Column Text
class TwoColumnLine<Size> extends CoreText<Size> {
  static const String type = "TWO_COLUMN";

  final String leftText;
  final String rightText;

  const TwoColumnLine({
    required this.leftText,
    required this.rightText,
    bool isBold = false,
    bool isItalic = false,
    String align = 'LEFT',
    required Size size,
  }) : super(
          size: size,
          align: align,
          isBold: isBold,
          isItalic: isItalic,
        );

  @override
  Map<String, dynamic> toMap() {
    return {
      "type": type,
      "leftText": leftText,
      "rightText": rightText,
      ...super.toMap(),
    };
  }
}

///Line separator
class Separator extends Printerble {
  static const String type = "SEPARATOR";

  const Separator();

  @override
  Map<String, dynamic> toMap() {
    return {
      "type": type,
    };
  }
}

/// For paper feeding.
class FeedPaper extends Printerble {
  static const String type = "FEED";

  /// number of [lines] to feed
  final int lines;

  const FeedPaper({
    required this.lines,
  });

  @override
  Map<String, dynamic> toMap() {
    return {
      "type": type,
      "lines": lines,
    };
  }
}
